@@ -1,6 +1,8 @@
 #ifndef IMAGER_IMEXT_H_
 #define IMAGER_IMEXT_H_
 
+#include <stdlib.h>
+#include <errno.h>
 #include "imexttypes.h"
 #include "immacros.h"
 
@@ -52,14 +54,11 @@ extern im_ext_funcs *imager_function_ext_table;
 
 #endif
 
-#define im_img_8_new(ctx, xsize, ysize, channels) ((im_extt->f_im_img_8_new)((ctx), (xsize), (ysize), (channels)))
-#define im_img_16_new(ctx, xsize, ysize, channels) ((im_extt->f_im_img_16_new)((ctx), (xsize), (ysize), (channels)))
-#define im_img_double_new(ctx, xsize, ysize, channels) ((im_extt->f_im_img_double_new)((ctx), (xsize), (ysize), (channels)))
 #define im_img_pal_new(ctx, xsize, ysize, channels, maxpal) ((im_extt->f_im_img_pal_new)((ctx), (xsize), (ysize), (channels), (maxpal)))
 
 #define i_img_destroy(im) ((im_extt->f_i_img_destroy)(im))
 #define i_sametype(im, xsize, ysize) ((im_extt->f_i_sametype)((im), (xsize), (ysize)))
-#define i_sametype_chans(im, xsize, ysize, channels) ((im_extt->f_i_sametype_chans)((im), (xsize), (ysize), (channels)))
+#define i_sametype_chans_extra(im, xsize, ysize, channels, extra) ((im_extt->f_i_sametype_chans_extra)((im), (xsize), (ysize), (channels), (extra)))
 #define i_img_info(im, info) ((im_extt->f_i_img_info)((im), (info)))
 
 #ifndef IMAGER_DIRECT_IMAGE_CALLS
@@ -68,14 +67,6 @@ extern im_ext_funcs *imager_function_ext_table;
 
 #if !IMAGER_DIRECT_IMAGE_CALLS
 
-#define i_ppix(im, x, y, val) ((im_extt->f_i_ppix)((im), (x), (y), (val)))
-#define i_gpix(im, x, y, val) ((im_extt->f_i_gpix)((im), (x), (y), (val)))
-#define i_ppixf(im, x, y, val) ((im_extt->f_i_ppixf)((im), (x), (y), (val)))
-#define i_gpixf(im, x, y, val) ((im_extt->f_i_gpixf)((im), (x), (y), (val)))
-#define i_plin(im, l, r, y, val) ((im_extt->f_i_plin)((im), (l), (r), (y), (val)))
-#define i_glin(im, l, r, y, val) ((im_extt->f_i_glin)((im), (l), (r), (y), (val)))
-#define i_plinf(im, l, r, y, val) ((im_extt->f_i_plinf)((im), (l), (r), (y), (val)))
-#define i_glinf(im, l, r, y, val) ((im_extt->f_i_glinf)((im), (l), (r), (y), (val)))
 #define i_gsamp(im, l, r, y, samps, chans, count) \
   ((im_extt->f_i_gsamp)((im), (l), (r), (y), (samps), (chans), (count)))
 #define i_gsampf(im, l, r, y, samps, chans, count) \
@@ -83,10 +74,10 @@ extern im_ext_funcs *imager_function_ext_table;
 
 #endif
 
-#define i_gsamp_bits(im, l, r, y, samps, chans, count, bits) \
-  (((im)->i_f_gsamp_bits) ? ((im)->i_f_gsamp_bits)((im), (l), (r), (y), (samps), (chans), (count), (bits)) : -1)
-#define i_psamp_bits(im, l, r, y, samps, chans, count, bits) \
-  (((im)->i_f_psamp_bits) ? ((im)->i_f_psamp_bits)((im), (l), (r), (y), (samps), (chans), (count), (bits)) : -1)
+#define i_plin(im, l, r, y, val) ((im_extt->f_i_plin)((im), (l), (r), (y), (val)))
+#define i_plinf(im, l, r, y, val) ((im_extt->f_i_plinf)((im), (l), (r), (y), (val)))
+#define i_glin(im, l, r, y, val) ((im_extt->f_i_glin)((im), (l), (r), (y), (val)))
+#define i_glinf(im, l, r, y, val) ((im_extt->f_i_glinf)((im), (l), (r), (y), (val)))
 
 #define i_new_fill_solid(c, combine) ((im_extt->f_i_new_fill_solid)((c), (combine)))
 #define i_new_fill_solidf(c, combine) ((im_extt->f_i_new_fill_solidf)((c), (combine)))
@@ -160,7 +151,7 @@ extern im_ext_funcs *imager_function_ext_table;
 #define i_flood_cfill_border(im, seedx, seedy, fill, border) ((im_extt->f_i_flood_cfill_border)((im), (seedx), (seedy), (fill), (border)))
 
 #define i_poly_aa_m(im, count, x, y, mode, c) ((im_extt->f_i_poly_aa_m)((im), (count), (x), (y), (mode), (c)))
-#define i_poly_aa_cfill_m(im, count, x, y, mode, fill) ((im_extt->f_i_poly_aa_m)((im), (count), (x), (y), (mode), (fill)))
+#define i_poly_aa_cfill_m(im, count, x, y, mode, fill) ((im_extt->f_i_poly_aa_cfill_m)((im), (count), (x), (y), (mode), (fill)))
 #define i_poly_poly_aa(im, count, polys, mode, c) ((im_extt->f_i_poly_poly_aa)((im), (count), (polys), (mode), (c)))
 #define i_poly_poly_aa_cfill(im, count, polys, mode, fill) ((im_extt->f_i_poly_poly_aa_cfill)((im), (count), (polys), (mode), (fill)))
 
@@ -181,16 +172,13 @@ extern im_ext_funcs *imager_function_ext_table;
 
 #define i_img_setmask(img, mask) ((im_extt->f_i_img_setmask)((img), (mask)))
 #define i_img_getmask(img) ((im_extt->f_i_img_getmask)(img))
-#define i_img_getchannels(img) ((im_extt->f_i_img_getchannels)(img))
-#define i_img_get_width(img) ((im_extt->f_i_img_get_width)(img))
-#define i_img_get_height(img) ((im_extt->f_i_img_get_height)(img))
 #define i_lhead(file, line) ((im_extt->f_i_lhead)((file), (line)))
 #define i_loog (im_extt->f_i_loog)
 #define im_lhead(ctx, file, line) ((im_extt->f_im_lhead)((ctx), (file), (line)))
 #define im_loog (im_extt->f_im_loog)
 
 #define im_img_alloc(ctx) ((im_extt->f_im_img_alloc)(ctx))
-#define im_img_init(ctx, img) ((im_extt->fm_i_img_init)((ctx), (img)))
+#define im_img_init(ctx, img) ((im_extt->f_im_img_init)((ctx), (img)))
 
 #define i_img_is_monochrome(img, zero_is_white) ((im_extt->f_i_img_is_monochrome)((img), (zero_is_white)))
 
@@ -247,12 +235,58 @@ extern im_ext_funcs *imager_function_ext_table;
 
 #define im_push_errorf (im_extt->f_im_push_errorf)
 
-#define i_img_alpha_channel(im, channel) ((im_extt->f_i_img_alpha_channel)((im), (channel)))
-#define i_img_color_model(im) ((im_extt->f_i_img_color_model)((im)))
-#define i_img_color_channels(im) ((im_extt->f_i_img_color_channels)((im)))
-
 #define im_decode_exif(im, data, len) ((im_extt->f_im_decode_exif)((im), (data), (len)))
 
+#define im_io_get_max_mmap_size(ctx) ((im_extt->f_im_io_get_max_mmap_size)(ctx))
+#define im_io_set_max_mmap_size(ctx, newsize) ((im_extt->f_im_io_set_max_mmap_size)((ctx), (newsize)))
+
+#define i_img_refcnt_inc(im) ((im_extt->f_i_img_refcnt_inc)(im))
+#define i_img_check_entries(im) ((im_extt->f_i_img_check_entries)(im))
+#define im_img_new(ctx, vtbl, xsize, ysize, channels, extra, bits) \
+  ((im_extt->f_im_img_new)((ctx), (vtbl), (xsize), (ysize), (channels), (extra), (bits)))
+#define im_img_8_new_extra(ctx, xsize, ysize, channels, extra) \
+  ((im_extt->f_im_img_8_new_extra)((ctx), (xsize), (ysize), (channels), (extra)))
+#define im_img_16_new_extra(ctx, xsize, ysize, channels, extra) \
+  ((im_extt->f_im_img_16_new_extra)((ctx), (xsize), (ysize), (channels), (extra)))
+#define im_img_double_new_extra(ctx, xsize, ysize, channels, extra) \
+  ((im_extt->f_im_img_double_new_extra)((ctx), (xsize), (ysize), (channels), (extra)))
+#define im_img_float_new_extra(ctx, xsize, ysize, channels, extra) \
+  ((im_extt->f_im_img_float_new_extra)((ctx), (xsize), (ysize), (channels), (extra)))
+#define im_lin_img_16_new_extra(ctx, xsize, ysize, channels, extra) \
+  ((im_extt->f_im_lin_img_16_new_extra)((ctx), (xsize), (ysize), (channels), (extra)))
+#define im_lin_img_double_new_extra(ctx, xsize, ysize, channels, extra) \
+  ((im_extt->f_im_lin_img_double_new_extra)((ctx), (xsize), (ysize), (channels), (extra)))
+#define i_new_image_data_alloc_def(im) \
+  ((im_extt->f_i_new_image_data_alloc_def)(im))
+#define i_new_image_data_alloc_free(im, releaseme) \
+  ((im_extt->f_i_new_image_data_alloc_free)((im), (releaseme)))
+#define myzmalloc(size) ((im_extt->f_myzmalloc)(size))
+#define myzmalloc_file_line(size, file, lineno) \
+  ((im_extt->f_myzmalloc_file_line)((size), (file), (lineno)))
+
+/* these are a bit special, mostly the APIs won't be called directly,
+   but used to initialize a vtable.
+*/
+#define i_gsampf_fp (im_extt->f_i_gsampf_fp)
+#define i_psampf_fp (im_extt->f_i_psampf_fp)
+#define i_gsamp_bits_fb (im_extt->f_i_gsamp_bits_fb)
+#define i_psamp_bits_fb (im_extt->f_i_psamp_bits_fb)
+#define i_img_data_fallback (im_extt->f_i_img_data_fallback)
+
+#define i_addcolors_forward (im_extt->f_i_addcolors_forward)
+#define i_getcolors_forward (im_extt->f_i_getcolors_forward)
+#define i_setcolors_forward (im_extt->f_i_setcolors_forward)
+#define i_colorcount_forward (im_extt->f_i_colorcount_forward)
+#define i_maxcolors_forward (im_extt->f_i_maxcolors_forward)
+#define i_findcolor_forward (im_extt->f_i_findcolor_forward)
+
+#define i_gslin_fallback (im_extt->f_i_gslin_fallback)
+#define i_gslinf_fallback (im_extt->f_i_gslinf_fallback)
+#define i_pslin_fallback (im_extt->f_i_pslin_fallback)
+#define i_pslinf_fallback (im_extt->f_i_pslinf_fallback)
+
+#define im_fatal (im_extt->f_im_fatal)
+
 #ifdef IMAGER_LOG
 #ifndef IMAGER_NO_CONTEXT
 #define mm_log(x) { i_lhead(__FILE__,__LINE__); i_loog x; } 
@@ -262,6 +296,8 @@ extern im_ext_funcs *imager_function_ext_table;
 #define mm_log(x)
 #endif
 
+#include "iminline.h"
+
 #ifdef __cplusplus
 }
 #endif
