@@ -4,37 +4,12 @@
 /* keep this file simple - apidocs.perl parses it. */
 
 #include "imdatatypes.h"
+#include "imapiver.h"
 #include <stdarg.h>
 
-/*
- IMAGER_API_VERSION is similar to the version number in the third and
- fourth bytes of TIFF files - if it ever changes then the API has changed
- too much for any application to remain compatible.
-
- Version 2 changed the types of some parameters and pointers.  A
- simple recompile should be enough in most cases.
-
- Version 3 changed the behaviour of some of the I/O layer functions,
- and in some cases the initial seek position when calling file
- readers.  Switching away from calling readcb etc to i_io_read() etc
- should fix your code.
-
- Version 4 added i_psamp() and i_psampf() pointers to the i_img
- structure.
-
- Version 5 changed the return types of i_get_file_background() and
- i_get_file_backgroundf() from void to int.
-
-*/
-#define IMAGER_API_VERSION 5
-
-/*
- IMAGER_API_LEVEL is the level of the structure.  New function pointers
- will always remain at the end (unless IMAGER_API_VERSION changes), and
- will result in an increment of IMAGER_API_LEVEL.
-*/
-
-#define IMAGER_API_LEVEL 9
+#ifdef __cplusplus
+extern "C" {
+#endif
 
 typedef struct {
   int version;
@@ -48,19 +23,12 @@ typedef struct {
   void  (*f_myfree_file_line)(void *p, char*file, int line);
   void* (*f_myrealloc_file_line)(void *p, size_t newsize, char* file,int line);
 
-  i_img *(*f_i_img_8_new)(i_img_dim xsize, i_img_dim ysize, int channels); /* SKIP */
-  i_img *(*f_i_img_16_new)(i_img_dim xsize, i_img_dim ysize, int channels);  /* SKIP */
-  i_img *(*f_i_img_double_new)(i_img_dim xsize, i_img_dim ysize, int channels); /* SKIP */
   i_img *(*f_i_img_pal_new)(i_img_dim xsize, i_img_dim ysize, int channels, int maxpal); /* SKIP */
   void (*f_i_img_destroy)(i_img *im);
   i_img *(*f_i_sametype)(i_img *im, i_img_dim xsize, i_img_dim ysize);
-  i_img *(*f_i_sametype_chans)(i_img *im, i_img_dim xsize, i_img_dim ysize, int channels);
+  i_img *(*f_i_sametype_chans_extra)(i_img *im, i_img_dim xsize, i_img_dim ysize, int channels, int extra);
   void (*f_i_img_info)(i_img *im, i_img_dim *info);
 
-  int (*f_i_ppix)(i_img *im, i_img_dim x, i_img_dim y, const i_color *val);
-  int (*f_i_gpix)(i_img *im, i_img_dim x, i_img_dim y, i_color *val);
-  int (*f_i_ppixf)(i_img *im, i_img_dim x, i_img_dim y, const i_fcolor *val);
-  int (*f_i_gpixf)(i_img *im, i_img_dim x, i_img_dim y, i_fcolor *val);
   i_img_dim (*f_i_plin)(i_img *im, i_img_dim l, i_img_dim r, i_img_dim y, const i_color *vals);
   i_img_dim (*f_i_glin)(i_img *im, i_img_dim l, i_img_dim r, i_img_dim y, i_color *vals);
   i_img_dim (*f_i_plinf)(i_img *im, i_img_dim l, i_img_dim r, i_img_dim y, const i_fcolor *vals);
@@ -161,9 +129,6 @@ typedef struct {
   /* IMAGER_API_LEVEL 3 functions */
   void (*f_i_img_setmask)(i_img *im, int ch_mask);
   int (*f_i_img_getmask)(i_img *im);
-  int (*f_i_img_getchannels)(i_img *im);
-  i_img_dim (*f_i_img_get_width)(i_img *im);
-  i_img_dim (*f_i_img_get_height)(i_img *im);
   void (*f_i_lhead)(const char *file, int line_number);
   void (*f_i_loog)(int level, const char *msg, ...) I_FORMAT_ATTR(2,3);
 
@@ -218,9 +183,6 @@ typedef struct {
   void (*f_io_glue_destroy)(i_io_glue_t *ig);
 
   /* IMAGER_API_LEVEL 8 */
-  i_img *(*f_im_img_8_new)(im_context_t ctx, i_img_dim xsize, i_img_dim ysize, int channels);
-  i_img *(*f_im_img_16_new)(im_context_t ctx, i_img_dim xsize, i_img_dim ysize, int channels);
-  i_img *(*f_im_img_double_new)(im_context_t ctx, i_img_dim xsize, i_img_dim ysize, int channels);
   i_img *(*f_im_img_pal_new)(im_context_t ctx, i_img_dim xsize, i_img_dim ysize, int channels, int maxpal);
 
   void (*f_im_clear_error)(im_context_t ctx);
@@ -266,13 +228,73 @@ typedef struct {
 			     const double *y, i_poly_fill_mode_t mode,
 			     i_fill_t *fill);
 
-  int (*f_i_img_alpha_channel)(i_img *im, int *channel);
-  i_color_model_t (*f_i_img_color_model)(i_img *im);
-  int (*f_i_img_color_channels)(i_img *im);
-
   /* IMAGER_API_LEVEL 10 functions will be added here */
+  int (*f_im_decode_exif)(i_img *im, const unsigned char *data, size_t length);
+
+  /* IMAGER_API_LEVEL 11 functions will be added here */
+  int (*f_im_io_set_max_mmap_size)(im_context_t, size_t);
+  size_t (*f_im_io_get_max_mmap_size)(im_context_t);
+  void (*f_i_img_refcnt_inc)(i_img *);
+  int (*f_i_img_check_entries)(i_img *im);
+  i_img * (*f_im_img_new)(im_context_t, const i_img_vtable *vtbl, i_img_dim xsize, i_img_dim ysize,
+                          int channels, int extra, i_img_bits_t bits);
+  i_img * (*f_im_img_8_new_extra)(im_context_t, i_img_dim x, i_img_dim y, int ch, int extra);
+  i_img * (*f_im_img_16_new_extra)(im_context_t, i_img_dim x, i_img_dim y, int ch, int extra);
+  i_img * (*f_im_img_double_new_extra)(im_context_t, i_img_dim x, i_img_dim y, int ch, int extra);
+  i_image_data_alloc_t * (*f_i_new_image_data_alloc_def)(i_img *im);
+  i_image_data_alloc_t * (*f_i_new_image_data_alloc_free)(i_img *im, void *releaseme);
+  void *(*f_myzmalloc)(size_t);
+  void *(*f_myzmalloc_file_line)(size_t, const char *, int);
+
+  /* fallback handlers for image vtables */
+  i_img_dim (*f_i_gsampf_fp)(i_img *im, i_img_dim l, i_img_dim r, i_img_dim y,
+                             i_fsample_t *samp, int const *chans, int chan_count);
+  i_img_dim (*f_i_psampf_fp)(i_img *im, i_img_dim l, i_img_dim r, i_img_dim y,
+                             const i_fsample_t *samp, int const *chans, int chan_count);
+
+  i_img_dim (*f_i_gsamp_bits_fb)(i_img *im, i_img_dim l, i_img_dim r, i_img_dim y,
+                                 unsigned *samps, const int *chans, int chan_count,
+                                 int bits);
+  i_img_dim (*f_i_psamp_bits_fb)(i_img *im, i_img_dim l, i_img_dim r, i_img_dim y,
+                                 const unsigned *samps, const int *chans,
+                                 int chan_count, int bits);
+  i_image_data_alloc_t *
+  (*f_i_img_data_fallback)(i_img *im, i_data_layout_t layout, i_img_bits_t bits,
+                           unsigned flags, void **pdata, size_t *psize, int *extra);
+  int (*f_i_addcolors_forward)(i_img *im, const i_color *colors, int count);
+  int (*f_i_getcolors_forward)(i_img *im, int i, i_color *color, int count);
+  int (*f_i_setcolors_forward)(i_img *im, int i, const i_color *color, int count);
+  int (*f_i_colorcount_forward)(i_img *im);
+  int (*f_i_maxcolors_forward)(i_img *im);
+  int (*f_i_findcolor_forward)(i_img *im, const i_color *color, i_palidx *entry);
+  i_img_dim
+  (*f_i_gslin_fallback)(i_img *im, i_img_dim x, i_img_dim r, i_img_dim y,
+                        i_sample16_t *samp, const int *chan, int chan_count);
+  i_img_dim
+  (*f_i_gslinf_fallback)(i_img *im, i_img_dim x, i_img_dim r, i_img_dim y,
+                         i_fsample_t *samp, const int *chan, int chan_count);
+  i_img_dim
+  (*f_i_pslin_fallback)(i_img *im, i_img_dim x, i_img_dim r, i_img_dim y,
+                        const i_sample16_t *samp, const int *chan, int chan_count);
+  i_img_dim
+  (*f_i_pslinf_fallback)(i_img *im, i_img_dim x, i_img_dim r, i_img_dim y,
+                         const i_fsample_t *samp, const int *chan, int chan_count);
+
+  i_img *
+  (*f_im_img_float_new_extra)(im_context_t, i_img_dim x, i_img_dim y, int ch, int extra);
+  i_img *
+  (*f_im_lin_img_16_new_extra)(im_context_t, i_img_dim x, i_img_dim y, int ch, int extra);
+  i_img *
+  (*f_im_lin_img_double_new_extra)(im_context_t, i_img_dim x, i_img_dim y, int ch, int extra);
+  void
+  (*f_im_fatal)(im_context_t, int exit_code, const char *format, ...);
+
 } im_ext_funcs;
 
 #define PERL_FUNCTION_TABLE_NAME "Imager::__ext_func_table"
 
+#ifdef __cplusplus
+}
+#endif
+
 #endif
